function [x,s,A,xdb,dbyA,hiscg] = pdasl1(X,Xt,y,opts,x0)
%-------------------------------------------------------------------------%
% Solving nonconvex problem                                               %
%      1/2||X*x-y||^2 + alpha/2||x||^2 + lambda ||x||_1                      %
% by a primal-dual active set algorithm                                   %
%                                                                         %
% INPUTS:                                                                 %
%     X   ---  matrix, normalized                                         %
%     Xt  ---  transpose of sampling matrix                               %
%     y   ---  data vector                                                %
%    opts --- structure contains                                          %
%         alpha ---  regularization parameter (default: 0)                %
%          lam  ---  regualrization paramter for sparsity                 %
%           mu  ---  stopping parameter                                   %
%        MaxIt  ---  maximum number of iterations (default: 1)            %
%     x0  ---  initial guess (default: 0)                                 %
%    debias   --- 1 get the result after debias, 0 not (default:1)        %
% OUTPUTS:                                                                %
%      x  ---  solution                                                   %                                                    
%      s  ---  size of active set                                         %
%     nIt ---  # of iter for the PDAS stop                                %
%   hiscg ---  history of cg iteration (when X is implicitly)             %
% (c) by Yuling Jiao (yulingjiaomath@whu.edu.cn)                          %
% Created on Oct 17, 2013                                                 %
%-------------------------------------------------------------------------%
lam = opts.lam;
al  = opts.alpha;
Xty = opts.Xty;
T = lam;
implicit = isa(X,'function_handle');
if implicit == 0
      hiscg = [];
      p  = length(x0);
      % initializing ...
      pd  = x0 + Xty - Xt*(X*x0);
      A  = find(abs(pd)>T);
      s = length(A);
      x = zeros(p,1);
      xdb = x;
      dA = lam*sign(pd(A));
      XtyA = Xty(A);
      dbyA = - XtyA;
      rhs = XtyA - dA;
       XA = X(:,A);
       G = XA'*XA + al*eye(length(A));
       x(A) = G\rhs;
       if opts.debias == 1
             xdb(A) = G\XtyA;
       else
            xdb = x;
            dbyA =   dbyA - dA;  
       end
else
   p = opts.p;
   hiscg.res   = [];
   hiscg.iter  = [];
   % initializing ...
   pd  = x0 + Xty - Xt(X(x0));
   ox = x0;
   A  = find(abs(pd)>T);
   s = length(A);
   x = zeros(p,1);
   xdb = x;
   dA = lam*sign(pd(A));
   XtyA = Xty(A);
   dbyA = - XtyA;
   rhs = XtyA - dA;
   option.Lt = Xt;
   option.subset = A;
   option.p = p;
   itcg = opts.cgiter; 
   [x(A),cgres,cgiter] = Subcg(X, rhs, opts.cgtol, itcg, ox,[], option);
   if opts.debias == 1
        xdb(A) = Subcg(X, XtyA, opts.cgtol, itcg, ox, [], option);
   else
           xdb = x;
            dbyA =   dbyA - dA;  
   end
      hiscg.res   = [hiscg.res,cgres];
      hiscg.iter  = [hiscg.iter,cgiter];
end