
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%        This file shows several examples of PDASC-l1                     %
%        Containing  1-d CS problem with three type sample matrix         %
%        and 2-d CS benchmark image  example                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc
clear all 
close all
warning off
addpath(genpath(fileparts(mfilename('fullpath'))));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                         1-d CS example                       %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% --------------------------------------------------------
% Generate data
% --------------------------------------------------------

%% Random Bernoulli 
% p = 32768;                         % length of solution  
% n = 2048;                          % number of samples 
% K = 128;                           % number of nonzero elements 
% ratio = 3;                         % range of value in x = 10^ratio;
% sigma = 1e-3;                      % noise varince
% matrixtype = 'bernoulli';          % sampling matrix type 
% seednum = 100;                     % seed number                       
% [X,Xt,y,ye,xe,Ae,out] = gendata(n,p,K,sigma,ratio,seednum,matrixtype);

%% Random Gaussian 
p = 16384;                         % length of solution  
n = 4096;                          % number of samples 
K = 256;                           % number of nonzero elements 
ratio = 4;                         % range of value in x = 10^ratio;
sigma = 1e-2;                      % noise varince
matrixtype = 'gauss';              % sampling matrix type 
seednum = 0;                       % seed number                       
[X,Xt,y,ye,xe,Ae,out] = gendata(n,p,K,sigma,ratio,seednum,matrixtype);



% %% Partial DCT
% p = 2^16;                          % length of solution  
% n = 2^14;                          % number of samples 
% K = 2^10;                          % number of nonzero elements 
% ratio = 2;                         % range of value in x = 10^ratio;
% sigma = 1e-2;                      % noise varince
% matrixtype = 'partdct';            % sampling matrix type 
% seednum = 100;                     % seed number                       
% [X,Xt,y,ye,xe,Ae,out] = gendata(n,p,K,sigma,ratio,seednum,matrixtype);

% --------------------------------------------------------
% Define operator
% --------------------------------------------------------
A_forward = X;
A_transpose = Xt;
% --------------------------------------------------------
% Set parameter in pdascl1
% --------------------------------------------------------
opts.alpha = 0;
opts.N     = 100;          
opts.Lmax  = 1;
opts.Lmin  = 1e-10;
opts.MaxIt = 1;
opts.mu    = 0.5; 
opts.del = norm(y-ye);
opts.debias = 1;   % 1 for get the reslut of debiase, 0 not. 
implicit = isa(X,'function_handle');
if implicit == 1
    opts.n = length(ye);
    opts.p = length(xe);
    opts.cgtol = opts.del*1e-2;
    opts.cgiter = 5; 
    n = opts.n;
    p = opts.p;
end
% --------------------------------------------------------
% Solve
% -------------------------------------------------------- 
tic,
[x,lam,ithist] = pdascl1(A_forward,A_transpose,y,opts); 
toc
% --------------------------------------------------------
% Show results 
% --------------------------------------------------------
 figure(1), plot(1:p,xe,'ko',1:p,x,'r*'), axis square  % support recovery
    legend('exact', 'estimate','Location','southeast')
    II = length(ithist.res);
    AmAe = zeros(II,1);
    AemA = zeros(II,1);
    for k = 1:II
       AmAe(k) = length(setdiff(find(ithist.x(:,k)),Ae));
       AemA(k) = length(setdiff(Ae,find(ithist.x(:,k))));
    end
    figure(2), 
    subplot(1,2,1),plot(1:II,AmAe,'--ro',1:II,AemA,'--b*','LineWidth',2),
    legend('|A^{k}-A|','|A-A^{k}|',2,'Location','best')
    xlabel('k','FontSize',16), set(gca,'FontSize',16)
    subplot(1,2,2),  semilogy(1:II, ithist.res,'r*','LineWidth',2), 
    xlabel('k','FontSize',16),
    set(gca,'FontSize',16)
    ylabel('Residual on the path')
    figure(3),plot(ithist.as,'r+','LineWidth',2), axis square
    xlabel('k','FontSize',16),
    set(gca,'FontSize',16)
    title('Active set size  on the path')
% compute error 
err_l2  = norm(x-xe)/norm(xe);
err_linf= norm(x-xe,inf);
A       = find(x);
lenthA  = length(A);
display(sprintf('regu. para. = %g, size of active set = %g, rel. l_2 error = %g, abs. l_inf error = %g',...
lam,lenthA,err_l2, err_linf))

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%                         2-d CS image example                 %%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % --------------------------------------------------------
% % Generate data
% % --------------------------------------------------------
% sigma = 1e-4;            % noise varince
% matrixtype = 'real2d';   % standard CS setting for Phatom reconstruction
% seednum = 0;             % seed number                       
% [X,Xt,y,ye,xe,Ae,out] = gendata([],[],[],sigma,[],seednum,matrixtype);
% % --------------------------------------------------------
% % Define operator A_forward and A_transpose
% % --------------------------------------------------------
% n = length(ye);
% p = length(xe);
% A_forward = @(x) X(x);
% A_transpose = @(x) Xt(x);
% % --------------------------------------------------------
% % Set parameter in pdascl1
% % --------------------------------------------------------
% opts.alpha = 0;
% opts.N     = 100;          
% opts.Lmax  = 1;
% opts.Lmin  = 1e-10;
% opts.MaxIt = 1;
% opts.mu    = 0.5; 
% opts.del = norm(y-ye);
% opts.n = n;
% opts.p = p;
% opts.cgtol = opts.del*1e-2;
% opts.cgiter = 2; 
% opts.matrixtype = matrixtype;
% opts.debias = 0;
% % --------------------------------------------------------
% % Solve
% % -------------------------------------------------------- 
% tic,
% [x,lam,ithist] = pdascl1(A_forward,A_transpose,y,opts); 
% toc
% % --------------------------------------------------------
% % Show results 
% % --------------------------------------------------------
% Ibar = out.Ibar;
% scale = max(max(out.I)) - min(min(out.I));
% xori = reshape(xe,[sqrt(length(xe)) sqrt(length(xe))]);
% xori = out.W(xori) +  Ibar;
% figure(1)
% imagesc(xori)
% colormap(gray)
% title('Original Image');
% figure(2)
% x2d = reshape(x,[sqrt(length(x)) sqrt(length(x))]);
% x2d = out.W(x2d) +  Ibar;
% imagesc(x2d)
% colormap(gray)
% psnrx = psnr(out.I,x2d,scale);
% title(['PDASC-l1, PSNR = ',num2str(psnrx,'%2.0f')]);
% box off
% axis off
% % compute error 
% err_l2  = norm(x-xe)/norm(xe);
% err_linf= norm(x-xe,inf);
% A       = find(x);
% lenthA  = length(A);
% display(sprintf('regu. para. = %g, size of active set = %g, rel. l_2 error = %g, abs. l_inf error = %g',...
% lam,lenthA,err_l2, err_linf))
% 
