clc
clear all 
close all
warning off
addpath(genpath(fileparts(mfilename('fullpath'))));
% --------------------------------------------------------
% Generate data
% --------------------------------------------------------
%% Random Bernoulli 
p = 4096;                          % length of solution  
n = 1024;                          % number of samples 
K = 32;                            % number of nonzero elements 
ratio = 3;                         % range of value in x = 10^ratio;
sigma = 1e-3;                      % noise varince
matrixtype = 'gauss';              % sampling matrix type 
seednum = 0;                       % seed number                       
[X,Xt,y,ye,xe,Ae,out] = gendata(n,p,K,sigma,ratio,seednum,matrixtype);
% --------------------------------------------------------
% Define operator
% --------------------------------------------------------
A_forward = X;
A_transpose = Xt;
% --------------------------------------------------------
% Set parameter in pdascl1
% --------------------------------------------------------
opts.alpha = 0;
opts.N     = 100;          
opts.Lmax  = 1;
opts.Lmin  = 1e-10;
opts.MaxIt = 1;
opts.mu    = 0.5; 
opts.del = 1*norm(y-ye);  %  Using  BIC by setting  opts.del = 0.
opts.debias = 0;          % 1 for  MDP, 0 for  DP(get larger supprot). 
implicit = isa(X,'function_handle');
if implicit == 1
    opts.n = length(ye);
    opts.p = length(xe);
    opts.cgtol = opts.del*1e-5;
    opts.cgiter = 10; 
    n = opts.n;
    p = opts.p;
end
% --------------------------------------------------------
% Solve
% -------------------------------------------------------- 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% If you  get your own data y = A_forward(x) + noise and use pdascl1 to   %
% recover x. You had better normalize A_forward by using                  %
%                  [A_forward,D] = normalize(A_forward).                  %
% Then run pdascl1 getting  an approximation x. And using D*x as a output.% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic,
[x,lam,ithist] = pdascl1(A_forward,A_transpose,y,opts); 
toc
% --------------------------------------------------------
% Show results 
% --------------------------------------------------------
 figure(1), plot(1:p,xe,'ko',1:p,x,'r*'), axis square  % support recovery
    legend('exact', 'estimate','Location','southeast')
    II = length(ithist.res);
    AmAe = zeros(II,1);
    AemA = zeros(II,1);
    for k = 1:II
       AmAe(k) = length(setdiff(find(ithist.x(:,k)),Ae));
       AemA(k) = length(setdiff(Ae,find(ithist.x(:,k))));
    end
    figure(2), 
    subplot(1,2,1),plot(1:II,AmAe,'--ro',1:II,AemA,'--b*','LineWidth',2),
    legend('|A^{k}-A|','|A-A^{k}|',2,'Location','best')
    xlabel('k','FontSize',16), set(gca,'FontSize',16)
    subplot(1,2,2),  semilogy(1:II, ithist.res,'r*','LineWidth',2), 
    xlabel('k','FontSize',16),
    set(gca,'FontSize',16)
    ylabel('Residual on the path')
    figure(3),plot(ithist.as,'r+','LineWidth',2), axis square
    xlabel('k','FontSize',16),
    set(gca,'FontSize',16)
    title('Active set size  on the path')
% compute error 
err_l2  = norm(x-xe)/norm(xe);
err_linf= norm(x-xe,inf);
A       = find(x);
lenthA  = length(A);
display(sprintf('regu. para. = %g, size of active set = %g, rel. l_2 error = %g, abs. l_inf error = %g',...
lam,lenthA,err_l2, err_linf))